/*  Project: Open Source Gmini Tools
    file: Breakpoint.cpp, single breakpoint data

    Copyright (C) 2004  Michael R. Donat <michael@donat.org>
         with contributions by gromit' and Treyqae

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <iostream>
using namespace std;
#include "Breakpoint.h"

const unsigned int Breakpoint::R = (1 << 0);
const unsigned int Breakpoint::W = (1 << 1); 
const unsigned int Breakpoint::X = (1 << 2);

/*
 * Breakpoint : generic class
 *
 */
Breakpoint::Breakpoint( brktype_t type ) {
	this->type = type;
	enable();
}
void Breakpoint::enable() {
	enabled = true;
}
void Breakpoint::disable() {
	enabled = false;
}
Breakpoint::brktype_t Breakpoint::get_type() {
	return type;
}

/*
 * BreakpointX : breakpoint on execution
 * 
 */
BreakpointX::BreakpointX( u22 address ) : Breakpoint( EXECUTION ) {
	this->address = address;
}
const void BreakpointX::Print() {
	cout << "BPX @ 0x" << hex << address << dec;
}
const bool  BreakpointX::is( const u22 address, unsigned int access ) {
	return enabled && 
	       (address == this->address);
}
/*
 * BreakpointX : breakpoint on memory
 * 
 */
BreakpointM::BreakpointM( u22 address, unsigned int access ) : Breakpoint( MEMORY ) {
	this->access = access;
	this->address = address;
}
const void BreakpointM::Print() {
	cout << "BPM @ 0x";
	cout << hex << address << " " << dec;
	cout << ((R & access == R) ? "R" : "");
	cout << ((W & access == W) ? "W" : "");
}
const bool BreakpointM::is( const u22 address, unsigned int access ) {
	return enabled && 
	       (address == this->address ) && 
	       (access & this->access);
}

/*
 * BreakpointR : breakpoint on memory range
 * 
 */
BreakpointR::BreakpointR( u22 start, u22 end, unsigned int access ) : Breakpoint( MEMORY_RANGE ) {
	this->start = start;
	this->end = end;
	this->access = access;
}
const void BreakpointR::Print() {
	cout << "BPR @ [ 0x";
	cout << hex << start << " - 0x" << hex << end << " ] " << dec;
	cout << ((R & access == R) ? "R" : "");
	cout << ((W & access == W) ? "W" : "");
	cout << ((X & access == X) ? "X" : "");
}
// bpr 0x76e57 0x3f0190 X
const bool BreakpointR::is( const u22 address, unsigned int access ) {
	return enabled &&
	       (address >= start) && 
	       (address <= end) && 
	       (access & this->access);
}
