/*  Project: Open Source Gmini Tools
    file: CPUCore.h, header file for CPUCore.cpp

    Copyright (C) 2004  Michael R. Donat <michael@donat.org>
         with contributions by gromit' and Treyqae

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef CPUCoreH
#define CPUCoreH
#include <map>

class CPUCore {
private:
	enum SPSRegister { SPCL_FIQ = 0, SPCH_FIQ = 1, SSR_FIQ = 2,
                      SPCL_IRQ = 4, SPCH_IRQ = 5, SSR_IRQ = 6,
                                                  SSR_SWI = 10 };
	enum errcode { SUCCESS, NOTIMPLEMENTED, INVALID, RESERVED,
                  OPSIZE, MEMORY, ABSJUMP, RELJUMPUP, RELJUMPDN,
                  READONLY, WRITEONLY, KNOWNDATA, OOPS, BREAKPOINT };
	static const char errcodestr[14][20];
	enum { OPHISTORY = 100 };
//	enum { DISABLED, ENABLED } state_t;
	enum reason_t { BREAK, REACHED, ERROR };
	MMU *memory;
	u16 R[16];
	u6  E[8];
	u22 PCC, SPC_FIQ, SPC_IRQ;
	u16 SR, SPSR[7];
	u24 X0, X1, Y0, Y1,                    // 24 bit
	    A, B, C, D,
	    SA, SI, SG, MACSR,
	    PL, PH, MA0L, MA0H, MA1L, MA1H;
	    u16 MSR0, MSR1, MSR2,              // 16 bit
	    RP0, RP1, RP2, RP3,
	    SD0, SD1, SD2, SD3,
	    SD0E, SD3E, RPD0, RPD1,
	    MC0, MC1;
	u48 P;                                 // 48 bit (PH PL)
	u52 MA0, MA1;                          // 52 bit (MSR1_MAi MAiH MAiL )
	bool EC[4];

	u16 oldR[16];
	u6  oldE[8];
	u22 oldPC, oldSPC_FIQ, oldSPC_IRQ;
	u16 oldSR, oldSPSR[7];

	bool flagT() { return SR & 0x8000; }
	void flagT( bool t ) {if(t) SR |= 0x8000;else SR &= 0x7fff;}
	bool flagPM() { return SR & 0x0040; }
	void flagPM( bool t ) {if(t) SR |= 0x0040;else SR &= 0xffbf;}
	bool flagZ1() { return SR & 0x0020; }
	void flagZ1( bool t ) {if(t) SR |= 0x0020;else SR &= 0xffdf;}
	bool flagZ0() { return SR & 0x0010; }
	void flagZ0( bool t ) {if(t) SR |= 0x0010;else SR &= 0xffef;}
	bool flagV() { return SR & 0x0008; }
	void flagV( bool t ) {if(t) SR |= 0x0008;else SR &= 0xfff7;}
	bool flagTE() { return SR & 0x0004; }
	void flagTE( bool t ) {if(t) SR |= 0x0004;else SR &= 0xfffb;}
	bool flagIE() { return SR & 0x0002; }
	void flagIE( bool t ) {if(t) SR |= 0x0002;else SR &= 0xfffd;}
	bool flagFE() {return SR & 0x0001;}
	void flagFE( bool t ) {if(t) SR |= 0x0001;else SR &= 0xfffe;}

	u16  r( int n );
	void r( int n, u16 val );
	u6   e( int n );
	void e( int n, u6 val );
	u22  a( int n );
	void a( int n, u22 val );
	void a( int n, u6 vale, u16 valr );
	u16  d( int n );
	void d( int n, u16 val );

	int usage[283];
	u32 steps;
	char dumpstring[65 * 10 + 1];
	char ophistorym[OPHISTORY];
	u22  ophistoryN[OPHISTORY];
	u22  ophistoryPC[OPHISTORY];
	u16  ophistory1[OPHISTORY];
	u16  ophistory2[OPHISTORY];
	u32  thisop, lastop;
	bool haveDebugBreak;
	u32 debugbreak;
	u32 debugstr;

	int  perfops;
	int  printop( u32 i, char* opstring );
	void updateops( errcode erc, int mnnum, u16 op1, u16 op2 = ZERO );
	void updatedump();
	int depth;
	int olddepth;
public:
	enum Versions { gm120pre,  gm120v120, gm120v130, gm120v140, gm120v150, 
	                gm120v170, gm120v180, gm120v190, gm120v193, gm120v1100, 
	                gm220v160, gm220v170, gm220v180, 
	                gmSPv110,  gmSPv120,  gmSPv123,  gmSPv130 };
	enum Overides { ovFlagT = 0, ovFlagPM, ovFlagZ1, ovFlagZ0, ovFlagV, ovFlagTE, ovFlagIE, ovFlagFE = 7,
	                ovR0 =  8, ovR1, ovR2,  ovR3,  ovR4,  ovR5,  ovR6,  ovR7  = 15,   
	                ovR8 = 16, ovR9, ovR10, ovR11, ovR12, ovR13, ovR14, ovR15 = 23,
	                ovE8 = 24, ovE9, ovE10, ovE11, ovE12, ovE13, ovE14, ovE15 = 31,
	                ovA8 = 32, ovA9, ovA10, ovA11, ovA12, ovA13, ovA14, ovA15 = 39 };
	void overide( Overides what, u32 value );
private:
	void doOveride();
	Versions version;
public:
	void setVersion( Versions ver ) { version = ver; }
	void dumpfunc();
	CPUCore( MMU *mem );
	void locatefunctions();
	void stats();
	void dumpregs();
	void dumpops( u32 n = 0 );
	void dump();
	void go( u22 address );
	errcode step();
	void walk( u22 address = 0xffffffff );
	void jog( u22 address = 0xffffffff );
	void run( u22 address = 0xffffffff );
private:
	void stopping( reason_t why, char *what );
	bool verify();
	errcode doStep();
	errcode delaySlot();
	errcode branch( s22 off, bool save = false, bool delay = false);
	errcode jump( u22 addr, bool save = false, bool delay = false );
	u16 getOP( bool u32 = false);
	void ungetOP();
	errcode doop( int mnnum, u16 op, u16 op2 = ZERO );
	int subSPSR( SPSRegister n );
	u16  sub2scomp( u16 val );

	errcode subADSBC( int n, u16 v2, bool c, bool m );
	errcode subAND( int n, u16 v2 );
	errcode opADC1( u16 op1 );              //19
	errcode opADC2( u16 op1, u16 op2 );     //52
	errcode opADD1( u16 op1 );              //17
	errcode opADD2( u16 op1 );              //0
	errcode opADD3( u16 op1, u16 op2 );     //49
	errcode opADD4( u16 op1 );              //138
	errcode opADD5( u16 op1, u16 op2 );     //50
	errcode opADD6( u16 op1 );              //134
	errcode opAND1( u16 op1 );              //21
	errcode opAND2( u16 op1 );              //66
	errcode opAND3( u16 op1, u16 op2 );     //54
	errcode opBIT( u16 op1 );               //72-75
	errcode opBNZD( u16 op1 );              //154
	errcode opBR( u16 op1 );                //156-161
	errcode opBRAEC( u16 op1 );             //151,153
	errcode opBREAK( u16 op1 );             //82
	errcode opBSRD( u16 op1 );              //150
	errcode opCLD( u16 op1 );               //162-163
	errcode opCLRSR( u16 op1 );             //78
	errcode opCMP1( u16 op1 );              //25-29
	errcode opCMP2( u16 op1, u16 op2 );     //58-62
	errcode opCMP3( u16 op1 );              //133
	errcode opCMPEQ1( u16 op1 );            //125
	errcode opCMPEQ2( u16 op1 );            //65
	errcode opCMPEQ3( u16 op1, u16 op2 );   //136
	errcode opCOM( u16 op1 );               //45-47
	errcode opCOP( u16 op1 );               //164+
	errcode opDECC( u16 op1 );              //44
	errcode opDT( u16 op1 );                //42
	errcode opEXT( u16 op1 );               //48
	errcode opINCC( u16 op1 );              //43
	errcode opJMP1( u16 op1 );              //37-40
	errcode opJMP2( u16 op1, u16 op2 );     //117-120
	errcode opLD1( u16 op1 );               //30
	errcode opLD2( u16 op1 );               //126
	errcode opLD3( u16 op1 );               //2
	errcode opLD4( u16 op1, u16 op2 );      //63
	errcode opLD5( u16 op1, u16 op2 );      //137
	errcode opLDRE( u16 op1 );              //123-124
	errcode opLDB1( u16 op1 );              //9,13
	errcode opLDB2( u16 op1, u16 op2 );     //129-130
	errcode opLDB3( u16 op1 );              //10,14
	errcode opLDB4( u16 op1 );              //70-71
	errcode opLDC( u16 op1 );               //121
	errcode opLDPC( u16 op1 );              //115
	errcode opLDSvR1( u16 op1 );            //89-91,93-95,98
	errcode opLDSvR2( u16 op1 );            //101-103,105-107,110
	errcode opLDSR( u16 op1 );              //83-84
	errcode opLDW1( u16 op1 );              //3-4
	errcode opLDW2( u16 op1 );              //5,7
	errcode opLDW3( u16 op1, u16 op2 );     //127-128
	errcode opLDW4( u16 op1 );              //6,8
	errcode opLDW5( u16 op1 );              //11,15
	errcode opLDW6( u16 op1, u16 op2 );     //131-132
	errcode opLDW7( u16 op1 );              //12,16
	errcode opMUL( u16 op1 );               //140-143
	errcode opNOP( u16 op1 );               //81
	errcode opOR1( u16 op1 );               //22
	errcode opOR2( u16 op1 );               //67
	errcode opOR3( u16 op1, u16 op2 );      //55
	errcode opPOP1( u16 op1 );              //144
	errcode opPOP2( u16 op1 );              //146
	errcode opPUSH1( u16 op1 );             //147
	errcode opPUSH2( u16 op1 );             //149
	errcode opRETD( u16 op1 );              //88
	errcode opRET_FIQ( u16 op1 );           //85
	errcode opRET_IRQ( u16 op1 );           //86
	errcode opRET_SWI( u16 op1 );           //87
	errcode opRL( u16 op1 );                //32
	errcode opRR( u16 op1 );                //31
	errcode opRRC( u16 op1 );               //33
	errcode opSBC1( u16 op1 );              //20
	errcode opSBC2( u16 op1, u16 op2 );     //53
	errcode opSETSR( u16 op1 );             //79
	errcode opSLB( u16 op1 );               //41
	errcode opSR( u16 op1 );                //35
	errcode opSRA( u16 op1 );               //36
	errcode opSRB( u16 op1 );               //34
	errcode opSUB1( u16 op1 );              //18
	errcode opSUB2( u16 op1 );              //1
	errcode opSUB3( u16 op1 );              //139
	errcode opSUB4( u16 op1, u16 op2 );     //51
	errcode opSUB5( u16 op1 );              //135
	errcode opSWI( u16 op1 );               //77
	errcode opSYS( u16 op1 );               //76
	errcode opTST1( u16 op1 );              //24
	errcode opTST2( u16 op1 );              //69
	errcode opTST3( u16 op1, u16 op2 );     //57
	errcode opTSTSR( u16 op1 );             //80
	errcode opXOR1( u16 op1 );              //23
	errcode opXOR2( u16 op1 );              //68
	errcode opXOR3( u16 op1, u16 op2 );     //56
	errcode opEABS( u16 op1 );              //268.9
	errcode opEADD1( u16 op1, u16 op2 );    //166.1  //////////CHECK THIS OUT!!!!
	errcode opEADD1( u16 op1 );             //237.1, 238.1, 239.1, 266
	errcode opEADD2( u16 op1 );             //188, 191, 196, 199, 204
	errcode opEADD3( u16 op1 );             //272.A, 272.C
	errcode opEADD4( u16 op1 );             //175, 177, 181, 184,
	errcode opEBK( u16 op1 );               //279
	errcode opECLD( u16 op1 );              //164
	errcode opECP1( u16 op1, u16 op2 );     //166.3  //////////CHECK THIS OUT!!!!
	errcode opECP1( u16 op1 );              //237.3, 238.3, 239.3, 267
	errcode opECP2( u16 op1 );              //206
	errcode opECR( u16 op1 );               //272.3
	errcode opEDECC( u16 op1 );             //268.7
	errcode opEDIVQ( u16 op1 );             //272.E
	errcode opEEXP( u16 op1 );              //268.E
	errcode opEEXPC( u16 op1 );             //268.F
	errcode opEFS16( u16 op1 );             //268.A
	errcode opEFS8( u16 op1 );              //268.C
	errcode opEFZ16( u16 op1 );             //268.B
	errcode opEFZ8( u16 op1 );              //268.D
	errcode opEINCC( u16 op1 );             //268.6
	errcode opELD1( u16 op1, u16 op2 );     //166.0, 169  /////CHECK THIS OUT!!!!
	errcode opELD1( u16 op1);               //210, 235, 237.0, 238.0, 239.0, 265
	errcode opELD2( u16 op1 );              //190, 193, 198, 201
	errcode opELD3( u16 op1 );              //272.8
	errcode opELD4( u16 op1 );              //179, 180, 183, 186
	errcode opELD5( u16 op1, u16 op2 );     //165, 167, 170  //CHECK THIS OUT!!!!
	errcode opELD5( u16 op1 );              //194, 202, 209, 211, 212, 213, 214, 226, 228,
	                                        //229, 230, 236, 240, 250, 251, 275, 276, 277, 278
	errcode opELD6( u16 op1 );              //219
	errcode opEMAD1( u16 op1 );             //256
	errcode opEMAD2( u16 op1 );             //171, 241, 252, 258
	errcode opEMAD3( u16 op1 );             //215
	errcode opEMAX1( u16 op1 );             //245, 247
	errcode opEMAX2( u16 op1 );             //207
	errcode opEMIN1( u16 op1 );             //246, 248
	errcode opEMIN2( u16 op1 );             //208
	errcode opEMLD1( u16 op1 );             //262
	errcode opEMLD2( u16 op1 );             //173, 243, 254, 260
	errcode opEMLD3( u16 op1 );             //217
	errcode opEMSB1( u16 op1 );             //257
	errcode opEMSB2( u16 op1 );             //172, 242, 253, 259
	errcode opEMSB3( u16 op1 );             //216
	errcode opEMUL1( u16 op1 );             //263
	errcode opEMUL2( u16 op1 );             //174, 244, 255, 261
	errcode opEMUL3( u16 op1 );             //218
	errcode opENEG( u16 op1 );              //268.8
	errcode opENMSK( u16 op1 );             //227
	errcode opENOP( u16 op1 );              //249
	errcode opER( u16 op1 );                //274
	errcode opERESR( u16 op1 );             //272.5
	errcode opERND( u16 op1 );              //272.2
	errcode opERPD( u16 op1 );              //271
	errcode opERPN( u16 op1, u16 op2 );     //168
	errcode opERPN( u16 op1 );              //269
	errcode opERPR( u16 op1 );              //264
	errcode opERPS( u16 op1 );              //270
	errcode opES( u16 op1 );                //274
	errcode opESAT( u16 op1 );              //272.4
	errcode opESD( u16 op1 );               //231, 232, 233, 234
	errcode opESEC( u16 op1 );              //280, 281, 282
	errcode opESFT( u16 op1 );              //222
	errcode opESFTA( u16 op1 );             //223
	errcode opESFTD( u16 op1 );             //225
	errcode opESFTL( u16 op1 );             //224
	errcode opESLA1( u16 op1 );             //268.1
	errcode opESLA2( u16 op1 );             //272.1
	errcode opESLA( u16 op1 );              //268.3
	errcode opESLC( u16 op1 );              //268.5
	errcode opESRA1( u16 op1 );             //268.0
	errcode opESRA2( u16 op1 );             //272.0
	errcode opESRA8( u16 op1 );             //268.2
	errcode opESRC( u16 op1 );              //268.4
	errcode opESUB1( u16 op1, u16 op2 );    //166.2  //////////CHECK THIS OUT!!!!
	errcode opESUB1( u16 op1 );             //237.2, 238.2, 239.2
	errcode opESUB2( u16 op1 );             //189, 192, 197, 200, 205
	errcode opESUB3( u16 op1 );             //272.B, 272.D
	errcode opESUB4( u16 op1 );             //182, 185, 176, 178
	errcode opETST( u16 op1 );              //273
	errcode opReserved( u16 op1 );          //64, 92, 96, 97, 99, 100, 104, 108, 109, 111, 112, 113,
	                                        //114, 116, 122, 145, 148, 152, 155, 187, 195, 203, 220,
	                                        //221, 272.6, 272.7, 272.9, 272.F
	errcode opInvalid( u16 op1 );
};
#endif





