/*  Project: Open Source Gmini Tools
    file: MMIO.cpp, abstraction of memory mapped IO

    Copyright (C) 2004  Michael R. Donat <michael@donat.org>
         with contributions by gromit' and Treyqae

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <iostream>
#include <fstream>
using namespace std;
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include "types.h"
#include "MMIO.h"

const char MMIO::mapped[400][10] = {
      "0x000"    , "OSCCON"   , "WTCON"    , "0x03"     , "BTCON"    , "BTCNT",
      "WDTEN"    , "WDTCON"   , "TACON"    , "TAPRE"    , "TADATAH"  , "TADATAL",
      "TACNTH"   , "TACNTL"   , "0x00e"    , "0x00f"    , "TBCON"    , "TBPRE",
      "TBDATAH"  , "TBDATAL"  , "TBCNTH"   , "TBCNTL"   , "0x016"    , "0x017",
      "TCCON"    , "TCPRE"    , "TCDATAH"  , "TCDATAL"  , "TCCNTH"   , "TCCNTL",
      "0x01e"    , "0x01f"    , "IRQ0H"    , "IRQ0L"    , "IMR0H"    , "IMR0L",
      "IPR0H"    , "IPR0L"    , "IIR0H"    , "0x027"    , "IRQ1H"    , "IRQ1L",
      "IMR1H"    , "IMR1L"    , "IPR1H"    , "IPR1L"    , "0x02e"    , "0x02f",
      "P0"       , "P1"       , "P2"       , "P3"       , "P4"       , "P5",
      "P6"       , "P7"       , "P8"       , "P9"       , "P10"      , "0x03b",
      "0x03c"    , "0x03d"    , "0x03e"    , "0x03f"    , "P0CON"    , "P1CON",
      "P2CONH"   , "P2CONL"   , "P3CONH"   , "P3CONL"   , "P3PUR"    , "0x047",
      "P5CON"    , "P5PUR"    , "P5INTMODH", "P5INTMODL", "P5INTCON" , "0x04d",
      "0x04e"    , "0x04f"    , "P4CON"    , "P4INTCON" , "P4INTMOD" , "P6CON",
      "P7CON"    , "P8CON"    , "P9CON"    , "P10CON"   , "SMCON"    , "ECCNT",
      "ECCH"     , "ECCL"     , "ECCX"     , "ECCCLR"   , "ECCRSTH"  , "ECCRSTL",
      "PPDATA"   , "PPCDATA"  , "PPSCON"   , "PPSTAT"   , "PPCONH"   , "PPCONL",
      "PPINTCONH", "PPINTCONL", "PPINTPNDH", "PPINTPNDL", "PPACKD"   , "0x06b",
      "0x06c"    , "0x06d"    , "0x06e"    , "0x06f"    , "SIOCON"   , "SIOPS",
      "SIODATA"  , "0x073"    , "ADDATAH"  , "ADDATAL"  , "ADCON"    , "0x077",
      "0x078"    , "0x079"    , "0x07a"    , "0x07b"    , "0x07c"    , "0x07d",
      "0x07e"    , "0x07f"    , "FUNADDR"  , "PWRMAN"   , "FRAMELO"  , "FRAMEHI",
      "INTREG"   , "INTENA"   , "EPINDEX"  , "0x087"    , "0x088"    , "EPDIR",
      "INCSR"    , "OUTCSR"   , "INMAXP"   , "OUTMAXP"  , "WRTCNTLO" , "WRTCNTHI",
      "EP0FIFO"  , "EP1FIFO"  , "EP2FIFO"  , "EP3FIFO"  , "0x094"    , "0x095",
      "0x096"    , "0x097"    , "0x098"    , "0x099"    , "0x09a"    , "0x09b",
      "0x09c"    , "0x09d"    , "0x09e"    , "0x09f"    , "IISCON0"  , "IISMODE0",
      "IISPTR0"  , "0x0a3"    , "IISCON1"  , "IISMODE1" , "IISPTR1"  , "0x0a7",
      "PLL0DATAH", "PLL0DATAL", "PLL0CON"  , "0x0ab"    , "PLL1DATAH", "PLL1DATAL",
      "PLL1CON"  , "0x0af"    , "LCON"     , "UCON"     , "USSR"     , "TBR",
      "RBR"      , "UBRDR"    , "UPEND"    , "0x0b7"    , "IICCON"   , "IICSR",
      "IICDATA"  , "IICADDR"  , "0x0bc"    , "IICCNT"   , "0x0be"    , "0x0bf",
      "BUF64"    , "0x0c1"    , "0x0c2"    , "0x0c3"    , "0x0c4"    , "0x0c5",
      "0x0c6"    , "0x0c7"    , "0x0c8"    , "0x0c9"    , "0x0ca"    , "0x0cb",
      "0x0cc"    , "0x0cd"    , "0x0ce"    , "0x0cf"    , "0x0d0"    , "0x0d1",
      "0x0d2"    , "0x0d3"    , "0x0d4"    , "0x0d5"    , "0x0d6"    , "0x0d7",
      "0x0d8"    , "0x0d9"    , "0x0da"    , "0x0db"    , "0x0dc"    , "0x0dd",
      "0x0de"    , "0x0df"    , "0x0e0"    , "0x0e1"    , "0x0e2"    , "0x0e3",
      "0x0e4"    , "0x0e5"    , "0x0e6"    , "0x0e7"    , "0x0e8"    , "0x0e9",
      "0x0ea"    , "0x0eb"    , "0x0ec"    , "0x0ed"    , "0x0ee"    , "0x0ef",
      "0x0f0"    , "0x0f1"    , "0x0f2"    , "0x0f3"    , "0x0f4"    , "0x0f5",
      "0x0f6"    , "0x0f7"    , "0x0f8"    , "0x0f9"    , "0x0fa"    , "0x0fb",
      "0x0fc"    , "0x0fd"    , "0x0fe"    , "0x0ff"    , "IDSC0H"   , "IDSC0L",
      "IDSC1H"   , "IDSC1L"   , "IDSC2H"   , "IDSC2L"   , "IDSC3H"   , "IDSC3L",
      "RANCON"   , "LFSR8"    , "LFSR16H"  , "LFSR16L"  , "INTMODE"  , "0x10d",
      "0x10e"    , "0x10f"    , "MIUSCFG"  , "MIUDCOM"  , "MIUDCFGH" , "MIUDCFGL",
      "MIUDCNTH" , "MIUDCNTL" , "0x116"    , "0x117"    , "0x118"    , "0x119",
      "0x11a"    , "0x11b"    , "0x11c"    , "0x11d"    , "0x11e"    , "0x11f",
      "DDMACOM"  , "DDMACFG"  , "0x122"    , "DDMAIADRX", "DDMAIADRH", "DDMAIADRL",
      "0x126"    , "DDMAEADRX", "DDMAEADRH", "DDMAEADRL", "DDMANUMH" , "DDMANUML",
      "DDMACNTH" , "DDMACNTL" , "DDMANUMX" , "DDMACNTX" , "YDMACOM"  , "YDMACFG",
      "YDMAIADRH", "YDMAIADRL", "0x134"    , "YDMAEADRX", "YDMAEADRH", "YDMAEADRL",
      "YDMANUMH" , "YDMANUML" , "YDMACNTH" , "YDMACNTL" , "0x13c"    , "0x13d",
      "0x13e"    , "0x13f"    , "CACHECON" , "ICBASEH"  , "ICBASEL"  , "DCBASEH",
      "DCBASEL"  , "0x145"    , "0x146"    , "0x147"    , "0x148"    , "0x149",
      "0x14a"    , "0x14b"    , "0x14c"    , "0x14d"    , "0x14e"    , "0x14f",
      "LCNTR0H"  , "LCNTR0L"  , "LCNTR1H"  , "LCNTR1L"  , "LCNTR2H"  , "LCNTR2L",
      "LCNTR3H"  , "LCNTR3L"  , "LCNTR4H"  , "LCNTR4L"  , "LCNTR5H"  , "LCNTR5L",
      "0x15c"    , "LDBAR1H"  , "LDBAR1M"  , "LDBAR1L"  , "0x160"    , "LDBAR2H",
      "LDBAR2M"  , "LDBAR2L"  , "0x164"    , "LDCAR1H"  , "LDCAR1M"  , "LDCAR1L",
      "0x168"    , "LDCAR2H"  , "LDCAR2M"  , "LDCAR2L"  , "0x16c"    , "LSR",
      "LRLUTRHH" , "LRLUTRHL" , "LRLUTRLH" , "LRLUTRLL" , "LGLUTRHH" , "LGLUTRHL",
      "LGLUTRLH" , "LGLUTRLL" , "LBLUTRH"  , "LBLUTRL"  , "LDP1_7"   , "LDP3_7",
      "LDP4_7"   , "LDP5_7"   , "LDP6_7"   , "0x17d"    , "LDP1_5"   , "LDP2_5",
      "LDP3_5"   , "LDP4_5"   , "LDP1_4"   , "LDP3_4"   , "LDP1_3"   , "0x185",
      "0x186"    , "0x187"    , "0x188"    , "0x189"    , "0x18a"    , "0x18b",
      "0x18c"    , "0x18d"    , "0x18e"    , "0x18f"    };

MMIO::MMIO( u22 baseaddress ) {
   base = baseaddress;
}
u8  MMIO::read8( u22 address ) {
   address -= base;
   ++ioaccess[address][0];
   u8 value = io[address];
   switch( address ) {
      case ADCON:
         value |= 0x08;
         break;
      case PLL0CON:
         value |= 0x02;
         break;
      case TACON:
         value &= 0xfe;
         break;
   }
   cout << "Read from (u8)io[" << mapped[address] << "]=" << hex << (u16)value << endl;
   return value;
}
u16 MMIO::read16( u22 address ) {
   address -= base;
   ++ioaccess[address][0];
   ++ioaccess[address + 1][0];
   u16 value = (u16)((io[address] << 16) | io[address]);
   cout << "Read from (u16)io[" << mapped[address] << "]=" << hex << value << endl;
   return value;
}
bool MMIO::write8( u22 address, u8 value ) {
   address -= base;
   ++ioaccess[address][1];
//   cout << "Write to io[" << mapped[address] << "]=(u8)" << hex << (u16)value << endl;
   io[address] = value;
   char buffer[20];
   buffer[0] = '\0';
   dump8( strcat( strcat( buffer, "./mmio/" ), mapped[address] ), value );
   return true;
}
bool MMIO::write16( u22 address, u16 value ) {
   address -= base;
   ++ioaccess[address][1];
   ++ioaccess[address + 1][1];
//   cout << "Write to io[" << mapped[address] << "]=(u16)" << hex << value << endl;
   io[address] = (u8)((value & 0xff00) >> 8);
   io[address + 1] = value && 0xff;
   char buffer[20];
   buffer[0] = '\0';
   dump16( strcat( strcat( buffer, "./mmio/" ), mapped[address] ), value );
   return true;
}
void MMIO::dump() {
   for( u32 i = 0; i < 0x190; ++i ) {
      if( (ioaccess[i][0] != 0) || (ioaccess[i][1] != 0) )
         printf( "%03x io[%s]=0x%02x R:%d W:%d\n", i, mapped[i], io[i], ioaccess[i][0], ioaccess[i][1] );
   }
}
bool MMIO::dump8( const char* datafile, u8 byte ) {
   ofstream data( datafile, ios::out | ios::binary | ios::app );
   if( !data ) {
      cout << "Could not open port file: " << datafile << endl;
      return false;
   }
   data.write( (char*)(&byte), 1 );
   data.close();
   return true;
}
bool MMIO::dump16( const char* datafile, u16 byte ) {
   ofstream data( datafile, ios::out | ios::binary | ios::app );
   if( !data ) {
      cout << "Could not open port file: " << datafile << endl;
      return false;
   }
   data.write( ((char*)(&byte))+1, 1 );
   data.write( (char*)(&byte), 1 );
   data.close();
   return true;
}

