/*  Project: Open Source Gmini Tools
    file: MMU.h, header file for MMU.cpp

    Copyright (C) 2004  Michael R. Donat <michael@donat.org>
         with contributions by gromit' and Treyqae

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef MMUH
#define MMUH
#include <vector>
#include "MMIO.h"

class MMU {
public:
	enum { MEMSIZE = 0x400000 };
	enum { MEMFIQLOCKED = 0x3F0190 };
private:
   
	enum { MAPWIDTH = 1024, MAPHEIGHT = MEMSIZE / MAPWIDTH, MAPCHANNELS = 3 };
	enum { RAMBASE  = 0x000000 };
	enum { RAMSIZE  = 0x800000 };
	enum { RAMPAGE  = 0x200000 };

	enum { CODEBASE = 0x000000 };
	enum { DATABASE = 0x080000 };
	enum { M1BASE   = 0x200000 };
	enum { M2BASE   = 0x210000 };
	enum { M3BASE   = 0x220000 };
	enum { M4BASE   = 0x230000 };
	enum { IOBASE   = 0x3f0000 };
	enum { ICBASE   = 0x3fc400 };

	enum { CODESIZE =  0x80000 };
	enum { DATASIZE =  0x80000 };
	enum { M1SIZE   =   0x8000 };
	enum { M2SIZE   =   0x8000 };
	enum { M3SIZE   =   0x4000 };
	enum { M4SIZE   =   0x4000 };
	enum { IOSIZE   =    0x190 };
	enum { ICSIZE   =   0x3fc0 };

	u8* memory;
	u8* memmap;
	u8* ram;
	MMIO *io;

	u8* mem( u22 address );
	u22 base( u22 address );
	u32 loadsize;
	u32 loadbase;
public:
	enum access { READ16, READ8, WRITE16, WRITE8, CODE32, 
	              CODE16, FIRM,  RJUMP,   AJUMP,  DATA };
	MMU();
	~MMU();
	u8  dread8( u22 address );
	u16 dread16( u22 address );
	bool dwrite8( u22 address, u8 value );
	bool dwrite16( u22 address, u16 value );
	u16 cread16( u22 address );
	bool cwrite16( u22 address, u16 value );
	const u32 find( const u8* value, u8 length );
	bool isData( u22 address );
	void dumpio() { io->dump(); }
	void mapData();
	u16 cread( u22 address, bool op32 = false ); // used only for retriving opcodes;
	bool cwrite( u22 address, u8 value ); // used only for loading firmware file
	void prncstr( u22 address, u8 length );
	bool load( const char* filename, u32 baseaddress );
	void findStrings();
	bool dumpmem( char* datafile );
	bool dumpram( char* datafile );
	bool dump( char* datafile );
	void addmap( u22 address, access type );
	const u32 imagebase() {return loadbase;}
	const u32 imagesize() {return loadsize;}
};
#endif
