/*  Project: Open Source Gmini Tools
    file: disasm.cpp, main procedure for gmdisasm

    Copyright (C) 2004  Michael R. Donat <michael@donat.org>
         with contributions by gromit' and Treyqae

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <string.h>
#include <iostream>
#include <fstream>
#include <stdarg.h>
#include <stdio.h>
//#ifndef GNU_LIBC
//#include "getopt.h" // yanked from GNU libc source

#include "types.h"
#include "util.h"
#include "decodeOP.h"
#include "formatOP.h"
#include "SymMap.h"
#include "CommentMap.h"
#define DISASMNAME "disasm"

void listwrite(ofstream &data, const char* format, ... ) {
   char buffer[1024 * 10];
   va_list vargs;
   va_start( vargs, format );
   vsprintf( buffer, format, vargs );
   va_end( vargs );
   data.write( buffer, strlen(buffer) );
}

void disasmTCC730( u8* image, u32 loadaddress, u32 size, char* listfile, char* mapfile, char* commentfile ) {
   u8* end = image + size + loadaddress;
   u32 ip = loadaddress;
   symbols = new SymMap();
   symbols->load( mapfile );
   comments = new CommentMap();
   comments->load( commentfile, loadaddress );
   comments->findStrings( image, size, loadaddress );
   while( (image + ip) < end ) {
      u16 op1 = bu16( image + ip - loadaddress);
      u16 op2 = bu16( image + ip - loadaddress + 2);
      int mnnum = decodeOP( op1 );
      formatOP( mnnum, op1, op2, ip + opsize(mnnum) );
      ip += opsize(mnnum);
   }
   ip = loadaddress;
   ofstream data( listfile, ios::out );
   if( !data ) {
      cout << "Could not open list file: " << listfile << endl;
      return;
   }
   while( (image + ip) < end ) {
      u16 op1 = bu16( image + ip - loadaddress);
      u16 op2 = bu16( image + ip - loadaddress + 2);
      int mnnum = decodeOP( op1 );
      char* inst = formatOP( mnnum, op1, op2, ip + opsize(mnnum) );
      const char* sym = symbols->get(ip);
      const char* comment = comments->get( ip );
      if( sym != NULL ) {
         if( comment != NULL )
            listwrite( data, "%s                                          ;%s\n", sym, comment );
         else
            listwrite( data, "%s:\n", sym );
      }
      if( mnnum < 164 ) {
         if( opsize(mnnum) == 2 )
            listwrite( data, "%06X:( %3u) %04X     : %s", ip, mnnum, op1, inst );
         else if( opsize(mnnum) == 4 )
            listwrite( data, "%06X:( %3u) %04X %04X: %s", ip, mnnum, op1, op2, inst );
      }
      else {
         if( opsize(mnnum) == 2 )
            listwrite( data, "%06X:(E%3u) %04X     : %s", ip, mnnum - 164, op1, inst );
         else if( opsize(mnnum) == 4 )
            listwrite( data, "%06X:(E%3u) %04X %04X: %s", ip, mnnum - 164, op1, op2, inst );
      }
      if( sym == NULL && comment != NULL )
         listwrite( data, "    ;%s\n", comment );
      else
         listwrite( data, "\n" );
      ip += opsize(mnnum);
   }
   data.close();
   symbols->dump( "../disasm.map" );
   delete symbols;
   comments->dump( "../disasm.tag" );
   delete comments;
}

int main( int argc, char** argv ) { 
   if( argc < 2 || argc > 3)
      printf("Usage: %s file.bin [file.map] [file.tag]", DISASMNAME );
   else {
      char* filebase = basename( argv[1] );
      int baselength = strlen( filebase );
      char* mapfile;
      char* commentfile;
      if( argc >= 3 )
         mapfile = argv[2];
      else {
         mapfile = new char[baselength + 5];
         sprintf( mapfile, "%s.%s", filebase, "map" );
      }
      if( argc == 4 )
         commentfile = argv[3];
      else {
         commentfile = new char[baselength + 5];
         sprintf( commentfile, "%s.%s", filebase, "tag" );
      }
      char* listfile = new char[baselength + 5];
      sprintf( listfile, "%s.%s", filebase, "lst" );
      delete filebase;
      u8 *bin;
      int size = loadFile( argv[1], &bin );
      if( size <= 0 ) return -1;
      u32 loadaddress = 0x6000;
      disasmTCC730( bin, loadaddress, size, listfile, mapfile, commentfile );
      delete listfile;
      if( argc >= 3 ) delete mapfile;
      if( argc == 4 ) delete commentfile;
   }
   return 0;
}


