%{
#include <iostream>
#include <stdio.h>
#include <ctype.h>
using namespace std;
#include "types.h"
#include "MMU.h"
#include "CPUCore.h"
#include "BreakpointList.h"

extern "C"
{
        int yyparse();
	int yyerror();
        int yylex();
	
        //int yywrap() { return 1; }
	//int parsecmd();
	//void startparser();
	//void stopparser();
}
extern FILE* yyin;
extern FILE* yyout;

#define PROMPT "gmemu> "

extern CPUCore cpu;
extern MMU memory;
extern bool cmdexit;

void display_help() {
//          12345678901234567890123456789012345678901234567890123456789012345678901234567890
   cout << "gmemu commands:" << endl;
   cout << "  help              -- displays this message" << endl;
   cout << "  quit              -- quit gmemu" << endl;
   cout << "execution:" << endl;
   cout << "  go                -- force PC back to start address" << endl;
   cout << "  go [address]      -- force PC to address specified" << endl;
   cout << "  step              -- step" << endl;
   cout << "  walk              -- walk" << endl;
   cout << "  walk [address]    -- walk until address specified" << endl;
   cout << "  jog               -- jog" << endl;
   cout << "  jog [address]     -- jog until address specified" << endl;
   cout << "  run               -- run" << endl;
   cout << "  run [address]     -- run until address specified" << endl;
   cout << "breakpoints:" << endl;
   cout << "  bpx [address]     -- sets a breakpoint at execution of the specified address" << endl;
   cout << "  bpm [address]     -- sets a breakpoint at memory access of specified address" << endl;
   cout << "  bpr [start] [end] -- sets a breakpoint for memory access within a range" << endl;
   cout << "  list              -- display a list of set breakpoints" << endl;
   cout << "  delete [number]   -- delete the specified breakpoint" << endl;
   cout << "  clear             -- delete all breakpoints" << endl;
   cout << "status display:" << endl;
   cout << "  registers         -- dump registers" << endl;
   cout << "  ops               -- dump ops" << endl;
   cout << "  ops [number]      -- dump specified number of ops" << endl;
   cout << "  mmio              -- dump io statistics" << endl;
   cout << "file writing:" << endl;
   cout << "  ram               -- dumps contents of memory to a file named ram.bin" << endl;
   cout << "  flash             -- dumps contents of flash to a file named flash.bin" << endl;
   cout << "  image             -- dumps raw image of interactions with memory" << endl;
}; 

%}

%union VALTYPE {
	unsigned int address;
	unsigned int number;
	int integer;
}

%token _BPM _BPR _BPX _CLEAR _DELETE _FLASH _GO _GOTO _HELP _IMAGE _JOG _LIST 
%token _MMIO _OPS _QUIT _RAM _REGS _RUN _STEP _WALK _ANYOTHER
%token _EOL
%token <address> _ADDRESS
%token <number> _NUMBER
%token _R _W _X
%token <integer> _INTEGER

%start Input

%%
Input:
	 /* */
	| Line { /*cout << PROMPT;*/ } Input
	;

Line:	  _EOL 
	| Command _EOL
	;			

Command:
	  _BPM _ADDRESS Access		{ BL.BPM($<address>2, $<integer>3);
					  return 0; }
	| _BPR _ADDRESS _ADDRESS Access	{ BL.BPR($<address>2, $<address>3, $<integer>4);
					  return 0; }
	| _BPX _ADDRESS			{ BL.BPX($<address>2);
					  return 0; }
	| _CLEAR			{ BL.clear();
					  return 0; }
	| _DELETE _NUMBER		{ BL.del($<number>2);
					  return 0; }
	| _FLASH			{ memory.dumpmem( "flash.bin" );
					  return 0; }
	| _GO _ADDRESS			{ cpu.go($<address>2);
					  return 0; }
	| _GO				{ cpu.go(0x6100);
					  return 0; }
	| _HELP				{ display_help();
					  return 0; }
	| _IMAGE			{ memory.dump( "memusage.raw" );
					  return 0; }
	| _JOG _ADDRESS			{ cpu.jog($<address>2);
					  return 0; }
	| _JOG				{ cpu.jog();
					  return 0; }
	| _LIST				{ BL.Print();
					  return 0; }
	| _MMIO				{ memory.dumpio();
					  return 0; }
	| _OPS _NUMBER			{ cpu.dumpops(100);
					  return 0; }
	| _OPS				{ cpu.dumpops(100);
					  return 0; }
	| _QUIT				{ cout << "exiting..." << endl;
					  cmdexit = true;
					  return 0; }
	| _RAM				{ memory.dumpram( "ram.bin" );
					  return 0; }
	| _REGS				{ cpu.dumpregs();
					  return 0; }
	| _RUN _ADDRESS			{ cpu.run($<address>2);
					  return 0; }
	| _RUN				{ cpu.run();
					  return 0; }
	| _STEP				{ cpu.step(); cpu.dumpops();
					  return 0; }
	| _WALK _ADDRESS		{ cpu.walk($<address>2);
					  return 0; }
	| _WALK				{ cpu.walk();
					  return 0; }
	;
Access:
	  { $<integer>$ = 0; }
	| Access _R { $<integer>$ = $<integer>1 | Breakpoint::R; }
	| Access _W { $<integer>$ = $<integer>1 | Breakpoint::W; }
	| Access _X { $<integer>$ = $<integer>1 | Breakpoint::X; }
 	;
	
%%

int yyerror(char *s) {
	cerr << "Syntax error" << endl;
	return 0;
}

int parsecmd() {
	cout << PROMPT;
	return yyparse();
}

void startparser() {
	cmdexit = false;
	yyin = stdin;
	yyout = stderr;
}

void stopparser() {
}

