/*  Project: Open Source Gmini Tools
    file: util.cpp, functions to interact with the system.

    Copyright (C) 2004  Michael R. Donat <michael@donat.org>
         with contributions by gromit' and Treyqae

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <iostream>
#include <fstream>
using namespace std;
#include <string.h>
#include "types.h"
#include "util.h"

u32 lu32( const u8* src ) {
	return (u32)((*(src + 0) <<  0 ) |
	             (*(src + 1) <<  8 ) |
	             (*(src + 2) << 16 ) |
	             (*(src + 3) << 24 ));
}

u32 bu32( const u8* src ) {
	return (u32)((*(src + 3) <<  0 ) |
	             (*(src + 2) <<  8 ) |
	             (*(src + 1) << 16 ) |
	             (*(src + 0) << 24 ));
}

u16 lu16( const u8* src ) {
	return (u16)((*(src + 0) <<  0 ) |
	             (*(src + 1) <<  8 ));
}

u16 bu16( const u8* src ) {
	return (u16)((*(src + 1) <<  0 ) |
	             (*(src + 0) <<  8 ));
}

u32 chksum32( const u8* src, u32 len ) {
	u32 sum = 0;
	const u8* end = src + len;
	while( src < end )
		sum = ( sum + *(src++) );
	return sum;
}

/* use strchr() or strrchr() instead */
int strrpos( const char* str, const char c ) {
	int i = strlen( str );
	while( i > 0 && str[i] != c ) --i;
	return i;
}

char* cstr( char* dest, const u8* src, u32 len ) {
	memcpy( dest, src, len );
	dest[len] = 0;
	return dest;
}

int loadFile( const char* filename, u8 **buf ) {
	ifstream data( filename, ios::in | ios::binary | ios::ate );
	if( !data ) {
		cout << "Could not open data file: " << filename << endl;
		return -1;
	}
	u32 length = data.tellg();
	data.seekg( 0, ios::beg );
	u8* buffer = new u8[length];
	data.read( (char*)buffer, length );
	data.close();
	*buf = buffer;
	return length;
}

int writeFile( const char* filename, const u8* buffer, u32 length ) {
	ofstream data( filename, ios::out | ios::binary | ios::trunc );
	if( !data ) {
		cout << "Could not open output file: " << filename << endl;
		return -1;
	}
	data.write( (char*)buffer, length );
	data.close();
	return 0;
}

// allocates a char array which must be deleted.
char* basename( char* fullname ) {
	int len = strrpos( fullname, '.' );
	char* filebase = new char[len + 1];
	strncpy( filebase, fullname, len );
	filebase[len] = '\0';
	return filebase;
}

// allocates a char array which must be deleted.
char* fileextension( const char* fullname ) {
	int pos = strrpos( fullname, '.' );
	int len = strlen(fullname) - pos;
	char* fileext = new char[ len + 1 ];
	strncpy( fileext, fullname + pos, len );
	fileext[len] = '\0';
	return fileext;
}
